@@ -14,6 +14,7 @@ from typing import (
     TypeVar,
     Union,
 )
+from urllib.parse import urljoin
 
 from globus_sdk import (
     AccessTokenAuthorizer,
@@ -98,7 +99,7 @@ def _all_vals_for_keys(
             )
         elif isinstance(v, list):
             for val in v:
-                if k in key_name_set and isinstance(v, str):
+                if k in key_name_set and isinstance(val, str):
                     val_set.add(val)
                 elif isinstance(val, dict):
                     val_set.update(
@@ -392,10 +393,12 @@ class FlowsClient(BaseClient):
             flow_administrators, kwargs, "administered_by", "administrators"
         )
         temp_body["subscription_id"] = subscription_id
-        temp_body["input_schema"] = input_schema
         # Remove None / empty list items from the temp_body
-        req_body = {k: v for k, v in temp_body.items() if v}
-        return self.put(f"/flows/{flow_id}", req_body, **kwargs)
+        data = {k: v for k, v in temp_body.items() if v}
+        # After removing false-y values, add the input schema.
+        if input_schema is not None:
+            data["input_schema"] = input_schema
+        return self.put(f"/flows/{flow_id}", data, **kwargs)
 
     def get_flow(self, flow_id: str, **kwargs) -> GlobusHTTPResponse:
         """
@@ -460,25 +463,35 @@ class FlowsClient(BaseClient):
 
         """
         self.authorizer = self.flow_management_authorizer
+
         params = {}
-        if roles is not None and len(roles) > 0:
-            params["filter_roles"] = ",".join(roles)
-        if role is not None:
+
+        # *role* takes precedence over *roles* (plural).
+        if role:
             params["filter_role"] = role
-            params.pop(
-                "filter_roles", None
-            )  # role takes precedence over roles (plural)
-        if marker is not None:
+        elif roles:
+            params["filter_roles"] = ",".join(roles)
+
+        # *marker* takes precedence over *per_page*.
+        if marker:
             params["pagination_token"] = marker
-        if per_page is not None and marker is None:
+        elif per_page:
             params["per_page"] = str(per_page)
-        if filters is not None:
-            params.update(filters)
+
         if orderings:
-            builder = []
-            for field, value in orderings.items():
-                builder.append(f"{field} {value}")
-            params["orderby"] = ",".join(builder)
+            params["orderby"] = ",".join(
+                f"{field} {value}" for field, value in orderings.items()
+            )
+
+        if filters:
+            # Prevent *filters* from overwriting reserved keys.
+            filters.pop("filter_role", None)
+            filters.pop("filter_roles", None)
+            filters.pop("orderby", None)
+            filters.pop("pagination_token", None)
+            filters.pop("per_page", None)
+            params.update(filters)
+
         return self.get("/flows", params=params, **kwargs)
 
     def delete_flow(self, flow_id: str, **kwargs) -> GlobusHTTPResponse:
@@ -496,7 +509,7 @@ class FlowsClient(BaseClient):
 
         :param flow_id: The UUID identifying the Flow's scope to lookup
         """
-        flow_url = f"{self.base_url}/flows/{flow_id}"
+        flow_url = urljoin(self.base_url, f"/flows/{flow_id}")
         return ActionClient.new_client(
             flow_url, authorizer=self.authorizer
         ).action_scope
@@ -510,7 +523,7 @@ class FlowsClient(BaseClient):
         if flow_scope is None:
             flow_scope = self.scope_for_flow(flow_id)
 
-        flow_url = f"{self.base_url}/flows/{flow_id}"
+        flow_url = urljoin(self.base_url, f"/flows/{flow_id}")
         return self.get_authorizer_callback(
             flow_url=flow_url,
             flow_scope=flow_scope,
@@ -558,13 +571,15 @@ class FlowsClient(BaseClient):
             authorizer_callback defined for the FlowsClient will be used.
         """
         authorizer = self._get_authorizer_for_flow(flow_id, flow_scope, kwargs)
-        flow_url = f"{self.base_url}/flows/{flow_id}"
+        flow_url = urljoin(self.base_url, f"/flows/{flow_id}")
         ac = ActionClient.new_client(flow_url, authorizer)
-        run_monitors = merge_lists(run_monitors, kwargs, "monitor_by")
-        run_managers = merge_lists(run_managers, kwargs, "manage_by")
 
-        kwargs.pop("monitor_by", None)
-        kwargs.pop("manage_by", None)
+        # Merge monitors and managers with aliases.
+        # If either list is empty it will be replaced with None
+        # to prevent empty lists from appearing in the JSON request.
+        run_monitors = merge_lists(run_monitors, kwargs, "monitor_by") or None
+        run_managers = merge_lists(run_managers, kwargs, "manage_by") or None
+
         if dry_run:
             path = flow_url + "/dry-run"
             return ac.run(
@@ -605,7 +620,7 @@ class FlowsClient(BaseClient):
             authorizer_callback defined for the FlowsClient will be used.
         """
         authorizer = self._get_authorizer_for_flow(flow_id, flow_scope, kwargs)
-        flow_url = f"{self.base_url}/flows/{flow_id}"
+        flow_url = urljoin(self.base_url, f"/flows/{flow_id}")
         ac = ActionClient.new_client(flow_url, authorizer)
         return ac.status(flow_action_id)
 
@@ -630,7 +645,7 @@ class FlowsClient(BaseClient):
             authorizer_callback defined for the FlowsClient will be used.
         """
         authorizer = self._get_authorizer_for_flow(flow_id, flow_scope, kwargs)
-        flow_url = f"{self.base_url}/flows/{flow_id}"
+        flow_url = urljoin(self.base_url, f"/flows/{flow_id}")
         ac = ActionClient.new_client(flow_url, authorizer)
         return ac.resume(flow_action_id)
 
@@ -654,7 +669,7 @@ class FlowsClient(BaseClient):
             authorizer_callback defined for the FlowsClient will be used.
         """
         authorizer = self._get_authorizer_for_flow(flow_id, flow_scope, kwargs)
-        flow_url = f"{self.base_url}/flows/{flow_id}"
+        flow_url = urljoin(self.base_url, f"/flows/{flow_id}")
         ac = ActionClient.new_client(flow_url, authorizer)
         return ac.release(flow_action_id)
 
@@ -678,7 +693,7 @@ class FlowsClient(BaseClient):
             authorizer_callback defined for the FlowsClient will be used.
         """
         authorizer = self._get_authorizer_for_flow(flow_id, flow_scope, kwargs)
-        flow_url = f"{self.base_url}/flows/{flow_id}"
+        flow_url = urljoin(self.base_url, f"/flows/{flow_id}")
         ac = ActionClient.new_client(flow_url, authorizer)
         return ac.cancel(flow_action_id)
 
@@ -744,24 +759,38 @@ class FlowsClient(BaseClient):
             OrderedDict if trying to apply multiple orderings.
 
         """
+
         params = {}
+
+        # *role* takes precedence over *roles* (plural).
         if role:
             params["filter_role"] = role
         elif roles:
             params["filter_roles"] = ",".join(roles)
+
+        # *marker* takes precedence over *per_page*.
+        if marker:
+            params["pagination_token"] = marker
+        elif per_page:
+            params["per_page"] = str(per_page)
+
         if statuses:
             params["filter_status"] = ",".join(statuses)
-        if marker is not None:
-            params["pagination_token"] = marker
-        if per_page is not None and marker is None:
-            params["per_page"] = str(per_page)
-        if filters is not None:
-            params.update(filters)
+
         if orderings:
-            builder = []
-            for field, value in orderings.items():
-                builder.append(f"{field} {value}")
-            params["orderby"] = ",".join(builder)
+            params["orderby"] = ",".join(
+                f"{field} {value}" for field, value in orderings.items()
+            )
+
+        if filters:
+            # Prevent *filters* from overwriting reserved keys.
+            filters.pop("filter_role", None)
+            filters.pop("filter_roles", None)
+            filters.pop("filter_status", None)
+            filters.pop("orderby", None)
+            filters.pop("pagination_token", None)
+            filters.pop("per_page", None)
+            params.update(filters)
 
         self.authorizer = self._get_authorizer_for_flow("", RUN_STATUS_SCOPE, kwargs)
         response = self.get(f"/runs", params=params, **kwargs)
@@ -812,9 +841,11 @@ class FlowsClient(BaseClient):
 
         :param  statuses: The same as in ``enumerate_runs``.
 
-        :param  roles: The same as in ``enumerate_runs``.
-            .. deprecated:: 0.12
-               Use ``role`` instead
+        :param roles:
+            ..  deprecated:: 0.12
+                Use ``role`` instead
+
+            The same as in ``enumerate_runs``.
 
         :param  marker: The same as in ``enumerate_runs``.
 
@@ -832,38 +863,50 @@ class FlowsClient(BaseClient):
             authorizer_callback defined for the FlowsClient will be used.
 
         """
+
         if flow_id is None:
             return self.enumerate_runs(
+                filters=filters,
+                marker=marker,
+                orderings=orderings,
+                per_page=per_page,
+                role=role,
                 roles=roles,
                 statuses=statuses,
-                marker=marker,
-                per_page=per_page,
-                filters=filters,
-                orderings=orderings,
-                role=role,
+                **kwargs,
             )
 
         params = {}
-        if statuses is not None and len(statuses) > 0:
-            params["filter_status"] = ",".join(statuses)
-        if roles is not None and len(roles) > 0:
-            params["filter_roles"] = ",".join(roles)
-        if role is not None:
+
+        # *role* takes precedence over *roles* (plural).
+        if role:
             params["filter_role"] = role
-            params.pop(
-                "filter_roles", None
-            )  # role takes precedence over roles (plural)
-        if marker is not None:
+        elif roles:
+            params["filter_roles"] = ",".join(roles)
+
+        # *marker* takes precedence over *per_page*.
+        if marker:
             params["pagination_token"] = marker
-        if per_page is not None and marker is None:
+        elif per_page:
             params["per_page"] = str(per_page)
-        if filters:
-            params.update(filters)
+
+        if statuses:
+            params["filter_status"] = ",".join(statuses)
+
         if orderings:
-            builder = []
-            for field, value in orderings.items():
-                builder.append(f"{field} {value}")
-            params["orderby"] = ",".join(builder)
+            params["orderby"] = ",".join(
+                f"{field} {value}" for field, value in orderings.items()
+            )
+
+        if filters:
+            # Prevent *filters* from overwriting reserved keys.
+            filters.pop("filter_role", None)
+            filters.pop("filter_roles", None)
+            filters.pop("filter_status", None)
+            filters.pop("orderby", None)
+            filters.pop("pagination_token", None)
+            filters.pop("per_page", None)
+            params.update(filters)
 
         self.authorizer = self._get_authorizer_for_flow(flow_id, flow_scope, kwargs)
         response = self.get(f"/flows/{flow_id}/actions", params=params, **kwargs)
@@ -948,7 +991,7 @@ class FlowsClient(BaseClient):
             authorizer_callback defined for the FlowsClient will be used.
         """
         authorizer = self._get_authorizer_for_flow(flow_id, flow_scope, kwargs)
-        flow_url = f"{self.base_url}/flows/{flow_id}"
+        flow_url = urljoin(self.base_url, f"/flows/{flow_id}")
         ac = ActionClient.new_client(flow_url, authorizer)
         return ac.log(flow_action_id, limit, reverse_order, marker, per_page)
 
